#include <R.h>  /* decl's of unif_rand(), GetRNGstate(), PutRNGstate() */
#include <Rmath.h>  /* for declaration of rexp() */

/* Simulate stochastic birth-death process with per-capita
 * birth rate phi and per-capita death rate mu.
 * eventTimes and popVec must be vectors of length s,
 * large enough to hold data for all of the time steps.
 */
void Cbirthdeath(double *phiptr, double *muptr,int *sptr,
	       double *eventTimes, int *popVec)
{
  int i, s = sptr[0];
  double phi = phiptr[0], mu = muptr[0], birthProb, r;

  birthProb = phi/(phi+mu);  /* probability a given event is a birth */
  eventTimes[0] = 0.0;  popVec[0] = 100;
  GetRNGstate();
  for (i=0; i < s-1; i++) {
    /* Inter-event time: generate a random value from exponential
     * distribution with mean 1/(popSize*(phi+mu)) */
    r = rexp(1.0/(popVec[i]*(phi+mu)));
    eventTimes[i+1] = eventTimes[i] + r;
    if (unif_rand() < birthProb)
      popVec[i+1] = popVec[i] + 1;  /* event is a birth */
    else
      popVec[i+1] = popVec[i] - 1;  /* event is a death */
  }
  PutRNGstate();
}
