#include <R.h>  /* decl's of unif_rand(), GetRNGstate(), PutRNGstate() */
#include <Rmath.h>  /* for declaration of rexp() */
#include <Rinternals.h>  /* for dealing with SEXP stuff */

/* Simulate stochastic birth-death process with per-capita
 * birth rate phi and per-capita death rate mu.
 * Return a list containing two vectors: eventTimes and popVec
 */
SEXP CbirthdeathExternal(SEXP args)
{
  int i, s=1000, *popVec;
  double phi=1.1, mu=1.0, birthProb, r, *eventTimes;
  SEXP returnList, eventTimesSXP, popVecSXP;

  args = CDR(args);  /* remove function name from args */
  Rprintf("length(args) is now %d\n", length(args));
  for (i=0; args != R_NilValue; i++, args = CDR(args)) {
    if (isNull(TAG(args)))
      error("Cbirthdeath(): unnamed arguments not allowed");
    /* extract name of next argument */
    const char *name = CHAR(PRINTNAME(TAG(args)));
    SEXP el = CAR(args);  /* and the value of that argument */
    if (length(el) != 1)
      error("Cbirthdeath(): All arguments must be scalars");
    if (!strcmp(name, "phi")) {
      if (TYPEOF(el) != REALSXP)
	error("Cbirthdeath(): phi must be a real value");
      phi = REAL(el)[0];
    } else if (!strcmp(name, "mu")) {
      if (TYPEOF(el) != REALSXP)
	error("Cbirthdeath(): mu must be a real value");
      mu = REAL(el)[0];
    } else if (!strcmp(name, "s")) {
      if (TYPEOF(el) != INTSXP)
	error("Cbirthdeath(): s must be an integer value");
      s = INTEGER(el)[0];
    } else {
      error("Cbirthdeath(): unrecognized argument name `%s'", name);
    }
  }
  Rprintf("phi=%g, mu=%g, s=%d\n", phi, mu, s);
  /* allocate memory for the two return vectors */
  PROTECT(eventTimesSXP=allocVector(REALSXP, s));
  PROTECT(popVecSXP=allocVector(INTSXP, s));
  /* then allocate memory for the list of length 2 */
  PROTECT(returnList=allocVector(VECSXP, 2));
  /* put the two vectors into the list */
  SET_VECTOR_ELT(returnList, 0, eventTimesSXP);
  SET_VECTOR_ELT(returnList, 1, popVecSXP);
  /* get pointers to the two vectors' actual data */
  eventTimes = REAL(eventTimesSXP);
  popVec = INTEGER(popVecSXP);

  birthProb = phi/(phi+mu);  /* probability a given event is a birth */
  eventTimes[0] = 0.0;  popVec[0] = 100;
  GetRNGstate();
  for (i=0; i < s-1; i++) {
    /* Inter-event time: generate a random value from exponential
     * distribution with mean 1/(popSize*(phi+mu)) */
    r = rexp(1.0/(popVec[i]*(phi+mu)));
    eventTimes[i+1] = eventTimes[i] + r;
    if (unif_rand() < birthProb)
      popVec[i+1] = popVec[i] + 1;  /* event is a birth */
    else
      popVec[i+1] = popVec[i] - 1;  /* event is a death */
  }
  PutRNGstate();
  UNPROTECT(3);  /* no more need to protect the 3 things we allocated */
  return(returnList);
}
