#include <R.h>   /* for declaration of random-number routines */
#include <Rinternals.h>  /* for dealing with SEXP stuff */
#include <math.h>   /* for declaration of floor() */

/* Permute the elements of a vector using the Fisher-Yates shuffle */
SEXP Cshufflecall(SEXP Rv, SEXP Rn)
{
  R_len_t i, j, n;
  double *v, *newv, tmpDbl;
  SEXP Rnewv;

  if (!isReal(Rv)) error("Vector v did not have real/double values");
  if (!isInteger(Rn)) error("n was not of type integer");
  v = REAL(Rv);  /* extract pointer to actual data for v */
  n = INTEGER(Rn)[0];  /* extract scalar value of n */
  if (n > length(Rv))  /* check n against length of vector v */
    error("Cshufflecall: n was %d, but vector v only had length %d\n",
	  n,length(Rv));
  /* allocate memory for return value, tell R not to garbage-collect it */
  PROTECT(Rnewv=allocVector(REALSXP, n));
  newv = REAL(Rnewv);  /* extract pointer to actual data for newv */
  
  GetRNGstate();
  for (i=0; i < n; i++)  /* copy original v to newv to work on it */
    newv[i] = v[i];
  for (i=n; i > 1; i--) {
    /* Generate a random integer from 0 to i-1
     * (not from 1 to i, because vectors in C use 0-based indexing). */
    j = (int)floor(unif_rand() * i);
    /* swap newv[j] with the last element of newv */
    tmpDbl = newv[j];  newv[j] = newv[i-1];  newv[i-1] = tmpDbl;
  }
  PutRNGstate();
  UNPROTECT(1);  /* we don't need to protect newv any more */
  return(Rnewv);
}
