#include "mex.h"
#include <stdlib.h>  /* for declaration of random() */
#include <math.h>   /* for declaration of log() */

/* 
 * phi, mu, s are input parameters.
 * eventTimes, popVec are used to return data
 */
void birthdeath(double phi, double mu, int s,
		double *eventTimes, double *popVec)
{
  int i;
  double u, r, birthProb;

  birthProb = phi/(phi+mu);
  eventTimes[0] = 0.0;  popVec[0] = 100;
  for (i=0; i < s-1; i++) {
    /* generate a random value from exponential distribution
     * with mean 1/(popSize*(phi+mu)) */
    /* First generate uniform (0,1] value, being careful
     * not to generate the value 0 */
    u = ((double) random()+1.0) / (double)(RAND_MAX+1.0);
    r = -1.0/(popVec[i]*(phi+mu))*log(u);
    if (random()/(double)RAND_MAX < birthProb)
      popVec[i+1] = popVec[i] + 1;
    else
      popVec[i+1] = popVec[i] - 1;
    eventTimes[i+1] = eventTimes[i] + r;
  }
}


/* The gateway function.
 * Arguments are phi, mu, steps
 * Return values are eventTimes, popVec
 */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
  int s;
  unsigned int rseed;
  double phi, mu, *eventTimes, *popVec;

  /* Check for proper number of arguments */
  if ((nrhs < 3) || (nrhs > 4))
    mexErrMsgIdAndTxt("RMatlab:birthdeath:nrhs", 
      "Three input args are needed, and 4th optional: phi mu s [seed]");
  if (nlhs != 2)
    mexErrMsgIdAndTxt("RMatlab:birthdeath:nlhs", 
		      "Two output values are needed");

  phi = mxGetScalar(prhs[0]);
  mu = mxGetScalar(prhs[1]);
  s = (int)mxGetScalar(prhs[2]);
  if (nrhs == 4) {
    rseed = (unsigned int)mxGetScalar(prhs[3]);
    srandom(rseed);
  }
  plhs[0] = mxCreateDoubleMatrix(1,s,mxREAL);
  plhs[1] = mxCreateDoubleMatrix(1,s,mxREAL);
  eventTimes = mxGetPr(plhs[0]);
  popVec = mxGetPr(plhs[1]);

  birthdeath(phi, mu, s, eventTimes, popVec);
}
