#include "mex.h"
#include <stdlib.h>  /* for declaration of random() */
#include <math.h>   /* for declaration of log() */


void birthdeath(double phi, double mu, int s, 
		double *eventTimes, double *popVec)
{
  int i;
  double u, r, birthProb, *expParmPtr;
  mxArray *expParmMxArray, *randValMxArray;

  birthProb = phi/(phi+mu);
  eventTimes[0] = 0.0;  popVec[0] = 100;
  expParmMxArray = mxCreateDoubleMatrix(1, 1, mxREAL);
  expParmPtr = mxGetPr(expParmMxArray);
  for (i=0; i < s-1; i++) {
    /* generate a random value from exponential distribution
     * with mean 1/(popSize*(phi+mu)) */
    expParmPtr[0] = 1.0/(popVec[i]*(phi+mu));
    mexCallMATLAB(1, &randValMxArray, 1, &expParmMxArray, "exprnd");
    r = mxGetScalar(randValMxArray);
    mxDestroyArray(randValMxArray);
    eventTimes[i+1] = eventTimes[i] + r;
    mexCallMATLAB(1, &randValMxArray, 0, NULL, "rand");
    r = mxGetScalar(randValMxArray);
    mxDestroyArray(randValMxArray);
    if (r < birthProb)
      popVec[i+1] = popVec[i] + 1;
    else
      popVec[i+1] = popVec[i] - 1;
  }
}


/* The gateway function */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
  int s;
  unsigned int rseed;
  double phi, mu, *eventTimes, *popVec;

  /* Check for proper number of arguments */
  if ((nrhs < 3) || (nrhs > 4))
    mexErrMsgIdAndTxt("RMatlab:birthdeath:nrhs", 
      "Three input args are needed, and 4th optional: phi mu s [seed]");
  if (nlhs != 2)
    mexErrMsgIdAndTxt("RMatlab:birthdeath:nlhs", 
		      "Two output values are needed");

  phi = mxGetScalar(prhs[0]);
  mu = mxGetScalar(prhs[1]);
  s = (int)mxGetScalar(prhs[2]);
  if (nrhs == 4) {
    rseed = (unsigned int)mxGetScalar(prhs[3]);
    srandom(rseed);
  }
  plhs[0] = mxCreateDoubleMatrix(1,s,mxREAL);
  plhs[1] = mxCreateDoubleMatrix(1,s,mxREAL);
  eventTimes = mxGetPr(plhs[0]);
  popVec = mxGetPr(plhs[1]);

  birthdeath(phi, mu, s, eventTimes, popVec);
}
