#include "mex.h"
#include <stdlib.h>  /* for declaration of random() */
#include <math.h>   /* for declaration of floor() */

/* Permute the elements of a vector using the Fisher-Yates shuffle */
void Cshuffle(double *outv, double *v, int n)
{
  int i, j;
  double tmpDbl, *imaxPtr, *retValPtr;
  mxArray *imaxMxArray, *retValMxArray;

  imaxMxArray = mxCreateDoubleMatrix(1,1, mxREAL);
  imaxPtr = mxGetPr(imaxMxArray);
  for (i=0; i < n; i++)
    outv[i] = v[i];
  for (i=n; i > 1; i--) {
    /* Generate a random integer from 0 to i-1
     * (not from 1 to i, because vectors in C use 0-based indexing). */
    *imaxPtr = (double)i;
    mexCallMATLAB(1, &retValMxArray, 1, &imaxMxArray, "randi");
    retValPtr = mxGetPr(retValMxArray);
    /* subtract 1 since randi gives vals from 1..i */
    j = (int)(*retValPtr) - 1;
    mxDestroyArray(retValMxArray);  /* free memory from mexCallMATLAB */
    /* swap outv[j] with the last element of outv */
    tmpDbl = outv[j];  outv[j] = outv[i-1];  outv[i-1] = tmpDbl;
  }
}


/* The gateway function */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
  int n;
  double *v, *outv;

  /* Check for proper number of arguments */
  if (nrhs == 1)
    /* if length of vector wasn't specified, use entire vector */
    n = mxGetNumberOfElements(prhs[0]);
  else {
    if (nrhs != 2)
      mexErrMsgIdAndTxt("RMatlab:Cshuffle2:nrhs",
			"At most two arguments needed");
    /* Make sure second argument is a real scalar */
    if (!mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) ||
	mxGetNumberOfElements(prhs[1]) != 1)
      mexErrMsgIdAndTxt("Cshuffle2:invalidArgument", 
			"Second argument must be a real scalar");
    /* Get length of vector */
    n = (int)mxGetScalar(prhs[1]);
  }

  if (nlhs > 1)
    mexErrMsgIdAndTxt("RMatlab:Cshuffle2:nlhs", "One return value needed");
  
  /* Make sure first argument is a vector, not a matrix */
  if (mxGetM(prhs[0]) != 1 && mxGetN(prhs[0]) != 1)
    mexErrMsgIdAndTxt("Cshuffle2:1stArgNotVector", 
		      "First argument must be a vector");
  
  /* Create pointer to data in input vector */
  v = mxGetPr(prhs[0]);

  /* Create row vector for return argument */
  plhs[0] = mxCreateDoubleMatrix(1,n,mxREAL);
  outv = mxGetPr(plhs[0]);

  Cshuffle(outv,v,n);
}
