#include "mex.h"

/* Two optional parameters can be given:
 *   n: how many exponentially-distributed random values to generate
 *   mu: their mean
 *   Default values: n=5, mu=2.0
 */
void
mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  double mu=2.0, *dblPtr;
  int n=5, i;
  mxArray *muArray, *sizeArray, *randValsMxArray, *rhs[2], *meanMxArray;

  if (nrhs >= 1) {
    n = (int)mxGetScalar(prhs[0]);
    if (nrhs == 2)
      mu = mxGetScalar(prhs[1]);
    if (nrhs > 2)
      mexErrMsgIdAndTxt("RMatlab:testmexcall:nrhs",
		      "One or two arguments needed: n mu");
  }
    
  muArray = mxCreateDoubleMatrix(1, 1, mxREAL);
  dblPtr = mxGetPr(muArray);
  dblPtr[0] = mu;
  sizeArray = mxCreateDoubleMatrix(1, 2, mxREAL);
  dblPtr = mxGetPr(sizeArray);
  dblPtr[0] = (double)n;  dblPtr[1] = 1.0;
  rhs[0] = muArray;  rhs[1] = sizeArray;
  mexCallMATLAB(1, &randValsMxArray, 2, rhs, "exprnd");
  dblPtr = mxGetPr(randValsMxArray);
  for (i=0; i < n; i++)
    mexPrintf("  value = %g\n", dblPtr[i]);
  mexCallMATLAB(1, &meanMxArray, 1, &randValsMxArray, "mean");
  dblPtr = mxGetPr(meanMxArray);
  mexPrintf("mean = %g\n", dblPtr[0]);
}
